from opengsq.protocol_base import ProtocolBase
from opengsq.protocol_socket import TcpClient


class TeamSpeak3(ProtocolBase):
    """TeamSpeak 3 Protocol"""
    full_name = 'TeamSpeak 3 Protocol'

    def __init__(self, host: str, port: int, voice_port: int, timeout: float = 5):
        super().__init__(host, port, timeout)
        self._voice_port = voice_port

    async def get_info(self):
        response = await self.__send_and_receive(b'serverinfo')
        return self.__parse_kvs(response)

    async def get_clients(self):
        response = await self.__send_and_receive(b'clientlist')
        return self.__parse_rows(response)

    async def get_channels(self):
        response = await self.__send_and_receive(b'channellist -topic')
        return self.__parse_rows(response)

    async def __send_and_receive(self, data: bytes):
        with TcpClient() as tcpClient:
            tcpClient.settimeout(self._timeout)
            await tcpClient.connect((self._host, self._port))

            # b'TS3\n\rWelcome to the TeamSpeak 3 ServerQuery interface,
            # type "help" for a list of commands and "help <command>" for information on a specific command.\n\r'
            await tcpClient.recv()

            # b'error id=0 msg=ok\n\r'
            tcpClient.send(f'use port={self._voice_port}\n'.encode())
            await tcpClient.recv()

            tcpClient.send(data + b'\x0A')
            response = b''

            while not response.endswith(b'error id=0 msg=ok\n\r'):
                response += await tcpClient.recv()

        # Remove last bytes b'\n\rerror id=0 msg=ok\n\r'
        return response[:-21]

    def __parse_rows(self, response: bytes):
        return [self.__parse_kvs(row) for row in response.split(b'|')]

    def __parse_kvs(self, response: bytes):
        kvs = {}

        for kv in response.split(b' '):
            items = kv.split(b'=', 1)
            key = str(items[0], encoding='utf-8', errors='ignore')
            val = str(items[1], encoding='utf-8', errors='ignore') if len(items) == 2 else ""
            kvs[key] = val.replace('\\p', '|').replace('\\s', ' ').replace('\\/', '/')

        return kvs


if __name__ == '__main__':
    import asyncio
    import json

    async def main_async():
        teamspeak3 = TeamSpeak3(host='145.239.200.2', port=10011, voice_port=9987, timeout=5.0)
        info = await teamspeak3.get_info()
        print(json.dumps(info, indent=None) + '\n')
        clients = await teamspeak3.get_clients()
        print(json.dumps(clients, indent=None) + '\n')
        channels = await teamspeak3.get_channels()
        print(json.dumps(channels, indent=None) + '\n')

    asyncio.run(main_async())



from opengsq.binary_reader import BinaryReader
from opengsq.exceptions import InvalidPacketException
from opengsq.protocol_socket import UdpClient
from opengsq.protocols.unreal2 import Unreal2


class KillingFloor(Unreal2):
    """Killing Floor Protocol"""
    full_name = 'Killing Floor Protocol'

    async def get_details(self):
        response = await UdpClient.communicate(self, b'\x79\x00\x00\x00' + bytes([self._DETAILS]))

        # Remove the first 4 bytes \x80\x00\x00\x00
        br = BinaryReader(response[4:])
        header = br.read_byte()

        if header != self._DETAILS:
            raise InvalidPacketException(
                'Packet header mismatch. Received: {}. Expected: {}.'
                .format(chr(header), chr(self._DETAILS))
            )

        details = {}
        details['ServerId'] = br.read_long()  # 0
        details['ServerIP'] = br.read_string()  # empty
        details['GamePort'] = br.read_long()
        details['QueryPort'] = br.read_long()  # 0
        details['ServerName'] = self._read_string(br)
        details['MapName'] = self._read_string(br)
        details['GameType'] = self._read_string(br)
        details['NumPlayers'] = br.read_long()
        details['MaxPlayers'] = br.read_long()
        details['WaveCurrent'] = br.read_long()
        details['WaveTotal'] = br.read_long()
        details['Ping'] = br.read_long()
        details['Flags'] = br.read_long()
        details['Skill'] = self._read_string(br)

        return details


if __name__ == '__main__':
    import asyncio
    import json

    async def main_async():
        # killingfloor
        killingFloor = KillingFloor(host='185.80.128.168', port=7708, timeout=10.0)
        details = await killingFloor.get_details()
        print(json.dumps(details, indent=None) + '\n')
        rules = await killingFloor.get_rules()
        print(json.dumps(rules, indent=None) + '\n')
        players = await killingFloor.get_players()
        print(json.dumps(players, indent=None) + '\n')

    asyncio.run(main_async())



import aiohttp
import time

from opengsq.protocol_base import ProtocolBase


class FiveM(ProtocolBase):
    """FiveM Protocol (https://docs.fivem.net/docs/server-manual/proxy-setup/)"""
    full_name = 'FiveM Protocol'

    async def _get(self, filename: str) -> dict:
        url = f'http://{self._host}:{self._port}/{filename}.json?v={int(time.time())}'

        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                return await response.json(content_type=None)

    async def get_info(self) -> dict:
        return await self._get('info')

    async def get_players(self) -> list:
        return await self._get('players')

    async def get_dynamic(self) -> dict:
        return await self._get('dynamic')


if __name__ == '__main__':
    import asyncio
    import json

    async def main_async():
        fivem = FiveM(host='144.217.10.12', port=30120, timeout=5.0)
        info = await fivem.get_info()
        print(json.dumps(info, indent=None) + '\n')
        players = await fivem.get_players()
        print(json.dumps(players, indent=None) + '\n')
        dynamic = await fivem.get_dynamic()
        print(json.dumps(dynamic, indent=None) + '\n')

    asyncio.run(main_async())
